#  tests for pycryptodome-3.12.0-py38h695f6f0_0 (this is a generated file);
print('===== testing package: pycryptodome-3.12.0-py38h695f6f0_0 =====');
print('running run_test.py');
#  --- run_test.py (begin) ---
from Crypto.Random import random


def test_pycryptodome_pkg():
    """
    Tests to validate the pycrptodome pkg.
    """
    print("Validating pycryptodome package")
    #  Returns  an integer <=a and <=b
    x = random.randint(0, 5000)
    y = random.randint(0,5000)
    assert 0 <= x <= 5000, "Failed to assert the random number from given range"
    assert x != y, "Failed to generate Random numbers"

    # Returns a random value from the list
    list1 = [1, 2, 3, 4, 5, 6]
    output_3 = random.choice(list1)
    assert output_3 in list1, "Failed to assert the random number from list Returned value is not in the list "


test_pycryptodome_pkg()#  --- run_test.py (end) ---

print('===== pycryptodome-3.12.0-py38h695f6f0_0 OK =====');
print("import: 'Crypto'")
import Crypto

print("import: 'Crypto.Cipher'")
import Crypto.Cipher

print("import: 'Crypto.Hash'")
import Crypto.Hash

print("import: 'Crypto.IO'")
import Crypto.IO

print("import: 'Crypto.Math'")
import Crypto.Math

print("import: 'Crypto.Protocol'")
import Crypto.Protocol

print("import: 'Crypto.PublicKey'")
import Crypto.PublicKey

print("import: 'Crypto.Random'")
import Crypto.Random

print("import: 'Crypto.SelfTest'")
import Crypto.SelfTest

print("import: 'Crypto.SelfTest.Cipher'")
import Crypto.SelfTest.Cipher

print("import: 'Crypto.SelfTest.Hash'")
import Crypto.SelfTest.Hash

print("import: 'Crypto.SelfTest.IO'")
import Crypto.SelfTest.IO

print("import: 'Crypto.SelfTest.Math'")
import Crypto.SelfTest.Math

print("import: 'Crypto.SelfTest.Protocol'")
import Crypto.SelfTest.Protocol

print("import: 'Crypto.SelfTest.PublicKey'")
import Crypto.SelfTest.PublicKey

print("import: 'Crypto.SelfTest.Random'")
import Crypto.SelfTest.Random

print("import: 'Crypto.SelfTest.Signature'")
import Crypto.SelfTest.Signature

print("import: 'Crypto.SelfTest.Util'")
import Crypto.SelfTest.Util

print("import: 'Crypto.Signature'")
import Crypto.Signature

print("import: 'Crypto.Util'")
import Crypto.Util

